----------------------------------------------------------------------------------
-- Company:     Fakulteta za elektrotehniko, Univerza v Ljubljani
-- Engineer:    Andrej Trost
-- 
-- Create Date:    09:04:21 03/26/2008 
-- Design Name: 
-- Module Name:    RxTx - Behavioral 
-- Project Name: 
-- Target Devices:  XC3S200 razvojni modul V1.1 in V2.0
-- Tool versions:  ISE 10.1
-- Description: 
--
-- Dependencies: 
--
-- Revision 0.02 - maj, 2009
-- Additional Comments: izboljsana sinhronizacija in popravljen oddajnik
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity RxTx is
    Port ( clk : in  STD_LOGIC;
           b1 : in  STD_LOGIC;
			  rx : in  STD_LOGIC;
           tx : out  STD_LOGIC;
           led1 : out  STD_LOGIC;
           led2 : out  STD_LOGIC);
end RxTx;

architecture Behavioral of RxTx is
signal div1: integer range 0 to 25000000;    -- delilnik za LED1 (50MHz clk)
signal blink: std_logic := '0';              -- stanje LED1

type Sstates is (Sidle, Sstart, Sdata, Sstop); -- RS232 oddajni avtomat
signal stx: Sstates;

type Rstates is (Ridle, Rstart, Rdata, Rstop);
signal srx: Rstates;

signal ce: std_logic;                        -- vzorcenje in
signal n: integer range 0 to 5208 := 0;      -- oddajni delilnik za RS232 (9600 baud)
signal b1not, tipka: std_logic;
signal bit_cnt: integer range 0 to 7 := 0;   -- stevec bitov
signal tx_reg: std_logic_vector(7 downto 0); -- oddajni pomikalni register

signal m: integer range 0 to 651 := 0;              -- sprejemni delilnik za RS232
signal cnt8: std_logic_vector(2 downto 0):="000";   -- 8x stevec za vzorcenje
signal rx1: std_logic;                              -- sinhronozacija rx
signal rxbit: std_logic := '1';                     -- notranji sinhroniziran rx signal
signal start: std_logic := '0';                     -- startaj oddajnik
signal sample: std_logic;                           -- trenutek vzorcenja
signal rx_cnt: integer range 0 to 7 := 0;           -- stevec bitov  
signal rx_reg: std_logic_vector(7 downto 0);        -- sprejemni pomikalni register

begin

-- delilnik za 2Hz utripanje LED
p_div: process(clk)
begin
  if rising_edge(clk) then
    if div1=25000000 then
	   div1 <= 0;
		blink <= not blink;
	 else
	   div1 <= div1 + 1;
	 end if;
  end if; 
end process;

led1 <= blink;

-------------------------------------------------------------------
-- oddajni delilnik in avtomat
-------------------------------------------------------------------

-- delitev vhodne ure s 5208 in generiranje bitne periode ce
P_timing: process(clk)
begin
 if rising_edge(clk) then   
   b1not <= not b1;         -- sinhronizacija in negacija tipke
	if n=5207 then              
	  n <= 0;
	  ce <= '1';
	  tipka <= b1not;
	else
	  n <= n + 1;
     ce <= '0';	  
	end if;
  end if;
end process;

-- oddajni avtomat
P_transmit: process(clk)
begin
 if rising_edge(clk) then
  case stx is
  
	when Sidle =>      -- start ob pritisku na tipko ali ob signalu 
	  tx <= '1';
	  if tipka='1' then 
	    tx_reg <= "01010011";  -- oddaj znak S
		 stx <= Sstart; 
	  elsif start='1' then
	    tx_reg <= rx_reg;      -- oddaj sprejeti podatek
		 stx <= Sstart;
	  end if;
	  
	when Sstart =>     -- nalozi pom. register in poslji startni bit
	  tx <= '1';
	  bit_cnt <= 0;
	  if ce='1' then
		 tx <= '0';            
	    stx <= Sdata;
	  end if;
	  
	when Sdata =>      -- pomikanje podatkovnih bitov (najprej LSB)
	  if ce='1' then  	  	    
	    tx <= tx_reg(bit_cnt);
		 if bit_cnt=7 then 
	      stx <= Sstop;
		 else
		   bit_cnt <= bit_cnt + 1;
	    end if;
	  end if;
	  
	when others =>    -- stop bit in cakaj, da bo tipka spuscena	  
	  if ce='1' then
	    tx <= '1';	  
		 if tipka='0' then  
	      stx <= Sidle;
		 end if;
	  end if;	
	  
  end case;	  
 end if;
end process;

-------------------------------------------------------------------
-- sprejemni delilnik in avtomat
-------------------------------------------------------------------

-- deli s 651
p_cnt: process(clk)
begin
 if rising_edge(clk) then
 	sample <= '0';
	rx1 <= rx;
	rxbit <= rx1;             -- 2x sinhronizacija vhoda
	
	if m=650 then             -- modul stetja 8x bitne frekvence
	  m <= 0;	  
	  if srx=Ridle then       -- 8x stevec naj bo v resetu ob Ridle
       cnt8 <= "000";	
	  else
	    cnt8 <= cnt8 + 1;
		 if cnt8="011" then  -- trenutek vzorcenja 
	      sample <= '1';
	    end if;
	  end if;	  
	else
	  m <= m + 1;
	end if;

 end if;
end process;

P_receive: process(clk)
begin
 if rising_edge(clk) then    
  case srx is 
   	
   when Ridle=>              -- zaceteno stanje, cakaj na startni bit
     start <= '0';
	  if rxbit='0' then 
	    srx <= Rstart;
	  end if;
	  
	when Rstart =>	          -- sprejet startni bit, resetiraj stevec bitov
	  if sample='1' then
	    srx <= Rdata;
		 rx_cnt <= 0;
	  end if;
	  
	when Rdata =>            -- sprejem podatkovnega paketa
	  if sample='1' then	    		   
		 rx_cnt <= rx_cnt + 1;
		 rx_reg(rx_cnt) <= rxbit;
		 if rx_cnt=7 then
		   srx <= Rstop;
		 end if;
	  end if;
	
   when Rstop =>	         -- preveri stop bit in sprejet podatek
	  if sample='1' then           
	    if rxbit='1' then      -- stop bit ok
		   start <= '1';			
		 end if;
		 srx <= Ridle;
	  end if;	  
	  
	when others =>
	  srx <= Ridle;
  end case;
 end if;
end process;

led2 <= '0' when srx=Rdata else '1';

end Behavioral;

